==> cursors_extended-neoforge-4.0.0%2B1.21.9-sources.jar: io/github/fishstiz/cursors_extended/util/DrawUtil.java <==
package io.github.fishstiz.cursors_extended.util;

import io.github.fishstiz.cursors_extended.cursor.Cursor;
import io.github.fishstiz.cursors_extended.gui.renderstate.GuiColoredRectRenderState;
import io.github.fishstiz.cursors_extended.platform.Services;
import io.github.fishstiz.cursors_extended.resource.CursorTexture;
import net.minecraft.Util;
import net.minecraft.client.gui.Font;
import net.minecraft.client.gui.GuiGraphics;
import net.minecraft.client.renderer.RenderPipelines;
import net.minecraft.network.chat.Component;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.Mth;
import org.joml.Matrix3x2f;

public class DrawUtil {
    private DrawUtil() {
    }

    public static void drawScrollableTextLeftAlign(
            GuiGraphics context,
            Font textRenderer,
            Component text,
            int startX,
            int startY,
            int endX,
            int endY,
            int color,
            boolean shadow
    ) {
        int textWidth = textRenderer.width(text);
        int textY = (startY + endY - 9) / 2 + 1;
        int availableWidth = endX - startX;

        if (textWidth > availableWidth) {
            int overflowWidth = textWidth - availableWidth;
            double timeSeconds = Util.getMillis() / 1000.0;
            double scrollDuration = Math.max(overflowWidth * 0.5, 3.0);
            double scrollFactor = Math.sin((Math.PI / 2) * Math.cos((Math.PI * 2) * timeSeconds / scrollDuration)) / 2.0 + 0.5;
            double scrollOffset = Mth.lerp(scrollFactor, 0.0, overflowWidth);

            context.enableScissor(startX, startY, endX, endY);
            context.drawString(textRenderer, text, startX - (int) scrollOffset, textY, color, shadow);
            context.disableScissor();
        } else {
            context.drawString(textRenderer, text, startX, textY, color, shadow);
        }
    }

    public static void drawScrollableTextLeftAlign(
            GuiGraphics context,
            Font textRenderer,
            Component text,
            int startX,
            int startY,
            int endX,
            int endY,
            int color
    ) {
        drawScrollableTextLeftAlign(context, textRenderer, text, startX, startY, endX, endY, color, true);
    }

    public static void drawCheckerboard(
            GuiGraphics guiGraphics,
            int x,
            int y,
            int width,
            int height,
            float cellWidth,
            float cellHeight,
            ResourceLocation checkerboard,
            int textureSize
    ) {
        float drawWidth = textureSize * cellWidth;
        float drawHeight = textureSize * cellHeight;

        float clippedWidth = Math.min(drawWidth, width);
        float clippedHeight = Math.min(drawHeight, height);

        float uRatio = clippedWidth / drawWidth;
        float vRatio = clippedHeight / drawHeight;

        int uvWidth = Math.round(textureSize * uRatio);
        int uvHeight = Math.round(textureSize * vRatio);

        guiGraphics.blit(
                RenderPipelines.GUI_TEXTURED,
                checkerboard,
                x, y,
                0, 0,
                (int) clippedWidth, (int) clippedHeight,
                uvWidth, uvHeight,
                textureSize, textureSize
        );
    }

    public static void renderOutline(GuiGraphics guiGraphics, int x, int y, int width, int height, int color) {
        guiGraphics.fill(x, y, x + width, y + 1, color);
        guiGraphics.fill(x, y + height - 1, x + width, y + height, color);
        guiGraphics.fill(x, y + 1, x + 1, y + height - 1, color);
        guiGraphics.fill(x + width - 1, y + 1, x + width, y + height - 1, color);
    }

    public static void fill(GuiGraphics guiGraphics, float minX, float minY, float maxX, float maxY, int color) {
        Services.PLATFORM.guiGraphicsHelper().submitGuiElementRenderState(
                guiGraphics,
                new GuiColoredRectRenderState(new Matrix3x2f(guiGraphics.pose()), minX, minY, maxX, maxY, color)
        );
    }

    public static void drawCursor(GuiGraphics guiGraphics, Cursor cursor, int x, int y, int size) {
        CursorTexture texture = cursor.getTexture();
        if (texture != null) {
            int spriteWidth = texture.spriteWidth();
            int spriteHeight = texture.spriteHeight();

            float scale = (float) size / Math.max(spriteWidth, spriteHeight);
            int drawWidth = Math.round(spriteWidth * scale);
            int drawHeight = Math.round(spriteHeight * scale);

            guiGraphics.blit(
                    RenderPipelines.GUI_TEXTURED,
                    texture.texturePath(),
                    x, y,
                    0, texture.spriteVOffset(),
                    drawWidth, drawHeight,
                    spriteWidth, spriteHeight,
                    texture.textureWidth(),
                    texture.textureHeight()
            );
        }
    }
}

==> cursors_extended-neoforge-4.1.4%2B1.21.9-sources.jar: io/github/fishstiz/cursors_extended/util/DrawUtil.java <==
package io.github.fishstiz.cursors_extended.util;

import io.github.fishstiz.cursors_extended.cursor.Cursor;
import io.github.fishstiz.cursors_extended.gui.renderstate.GuiColoredRectRenderState;
import io.github.fishstiz.cursors_extended.platform.Services;
import io.github.fishstiz.cursors_extended.resource.texture.CursorTexture;
import net.minecraft.Util;
import net.minecraft.client.gui.Font;
import net.minecraft.client.gui.GuiGraphics;
import net.minecraft.client.renderer.RenderPipelines;
import net.minecraft.network.chat.Component;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.Mth;
import org.joml.Matrix3x2f;

public class DrawUtil {
    private DrawUtil() {
    }

    public static void drawScrollableTextLeftAlign(
            GuiGraphics context,
            Font textRenderer,
            Component text,
            int startX,
            int startY,
            int endX,
            int endY,
            int color,
            boolean shadow
    ) {
        int textWidth = textRenderer.width(text);
        int textY = (startY + endY - 9) / 2 + 1;
        int availableWidth = endX - startX;

        if (textWidth > availableWidth) {
            int overflowWidth = textWidth - availableWidth;
            double timeSeconds = Util.getMillis() / 1000.0;
            double scrollDuration = Math.max(overflowWidth * 0.5, 3.0);
            double scrollFactor = Math.sin((Math.PI / 2) * Math.cos((Math.PI * 2) * timeSeconds / scrollDuration)) / 2.0 + 0.5;
            double scrollOffset = Mth.lerp(scrollFactor, 0.0, overflowWidth);

            context.enableScissor(startX, startY, endX, endY);
            context.drawString(textRenderer, text, startX - (int) scrollOffset, textY, color, shadow);
            context.disableScissor();
        } else {
            context.drawString(textRenderer, text, startX, textY, color, shadow);
        }
    }

    public static void drawScrollableTextLeftAlign(
            GuiGraphics context,
            Font textRenderer,
            Component text,
            int startX,
            int startY,
            int endX,
            int endY,
            int color
    ) {
        drawScrollableTextLeftAlign(context, textRenderer, text, startX, startY, endX, endY, color, true);
    }

    public static void drawCheckerboard(
            GuiGraphics guiGraphics,
            int x,
            int y,
            int width,
            int height,
            float cellWidth,
            float cellHeight,
            ResourceLocation checkerboard,
            int textureSize
    ) {
        float drawWidth = textureSize * cellWidth;
        float drawHeight = textureSize * cellHeight;

        float clippedWidth = Math.min(drawWidth, width);
        float clippedHeight = Math.min(drawHeight, height);

        float uRatio = clippedWidth / drawWidth;
        float vRatio = clippedHeight / drawHeight;

        int uvWidth = Math.round(textureSize * uRatio);
        int uvHeight = Math.round(textureSize * vRatio);

        guiGraphics.blit(
                RenderPipelines.GUI_TEXTURED,
                checkerboard,
                x, y,
                0, 0,
                (int) clippedWidth, (int) clippedHeight,
                uvWidth, uvHeight,
                textureSize, textureSize
        );
    }

    public static void renderOutline(GuiGraphics guiGraphics, int x, int y, int width, int height, int color) {
        guiGraphics.fill(x, y, x + width, y + 1, color);
        guiGraphics.fill(x, y + height - 1, x + width, y + height, color);
        guiGraphics.fill(x, y + 1, x + 1, y + height - 1, color);
        guiGraphics.fill(x + width - 1, y + 1, x + width, y + height - 1, color);
    }

    public static void fill(GuiGraphics guiGraphics, float minX, float minY, float maxX, float maxY, int color) {
        Services.PLATFORM.guiGraphicsHelper().submitGuiElementRenderState(
                guiGraphics,
                new GuiColoredRectRenderState(new Matrix3x2f(guiGraphics.pose()), minX, minY, maxX, maxY, color)
        );
    }

    public static void drawCursor(GuiGraphics guiGraphics, Cursor cursor, int x, int y, int size) {
        CursorTexture texture = cursor.getTexture();
        if (texture != null) {
            int spriteWidth = texture.spriteWidth();
            int spriteHeight = texture.spriteHeight();

            float scale = (float) size / Math.max(spriteWidth, spriteHeight);
            int drawWidth = Math.round(spriteWidth * scale);
            int drawHeight = Math.round(spriteHeight * scale);

            guiGraphics.blit(
                    RenderPipelines.GUI_TEXTURED,
                    texture.texturePath(),
                    x, y,
                    0, texture.spriteVOffset(),
                    drawWidth, drawHeight,
                    spriteWidth, spriteHeight,
                    texture.textureWidth(),
                    texture.textureHeight()
            );
        }
    }
}
